package com.perforb.jpaz.bootstrap;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;

import com.perforb.jpaz.settings.PropertiesSettings;

public class CommandParser
{
    /**
     * Singleton instance.
     */
    private static final CommandParser helper = new CommandParser();

    /**
     * Private Constructor.
     */
    private CommandParser()
    {
    }

    /**
     * Return the singleton instance.
     */
    public static CommandParser getInstance()
    {
        return helper;
    }

    public CommandLine parse(String[] args) throws ParseException
    {
        CommandLine line = null;
        Options options = null;
        try {
            CommandLineParser parser = new PosixParser();
            options = createOptions();
            line = parser.parse(options, args);
            check(line);
        } catch (ParseException e) {
            throw e;
        }
        return line;
    }

    public void usage()
    {
        HelpFormatter f = new HelpFormatter();
        f.setSyntaxPrefix("Usage: ");
        Map<String, String> messages = getMessages();
        f.printHelp(200, messages.get("cmd.line.syntax"), getMessages().get("cmd.line.header"),
                createOptions(), null);
    }

    private Options createOptions()
    {
        Map<String, String> messages = getMessages();
        Options options = new Options();
        options.addOption(null, "console", false, messages.get("console.description"));
        options.addOption(new Option(null, "csv", true, messages.get("csv.description")))
                .getOption("csv").setArgName("CSV_PATH");
        options.addOption(new Option(null, "charset", true, messages.get("charset.description")))
                .getOption("charset").setArgName("CHARSET");
        options.addOption(
                new Option("c", "classesdir", true, messages.get("classesdir.description")))
                .getOption("classesdir").setArgName("CLASSES_DIR");
        options.addOption(new Option("l", "libdir", true, messages.get("libdir.description")))
                .getOption("libdir").setArgName("LIB_DIR");
        options.addOption(
                new Option("r", "rootpackage", true, messages.get("rootpackage.description")))
                .getOption("rootpackage").setArgName("ROOT_PKG");
        options.addOption(new Option("o", "outlim", true, messages.get("outlim.description")))
                .getOption("outlim").setArgName("HEXADECIMAL_STRING");
        options.addOption("v", "version", false, messages.get("version.description"));
        options.addOption("h", "help", false, messages.get("help.description"));
        return options;
    }

    private void check(CommandLine line) throws ParseException
    {
        if (!line.hasOption("console") && !line.hasOption("csv")) {
            throw new ParseException("Missing option: --console or --csv");
        }
        String[] required = {"classesdir", "rootpackage",};
        List<String> missingList = new ArrayList<String>();
        for (String opt : required) {
            if (!line.hasOption(opt)) missingList.add(opt);
        }
        if (!missingList.isEmpty()) {
            String opts = "";
            for (String opt : missingList) {
                opts += "--" + opt + ", ";
            }
            throw new ParseException("Missing option: " + opts.substring(0, opts.lastIndexOf(',')));
        }
    }

    private Map<String, String> getMessages()
    {
        return PropertiesSettings.getInstance().load("messages.properties");
    }
}
