package com.perforb.jpaz.common;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * クラスパスを動的に追加するためのクラスです.
 * 
 * @author perforb
 */
public class ClassPathModifier
{
    private static final Class<?>[] PARAMETERS = new Class<?>[]{URL.class};

    /**
     * Private constructor.
     */
    private ClassPathModifier()
    {
    }

    /**
     * クラスパスを再帰的にたどりシステムクラスローダーに追加していきます.
     */
    public static void addClassPath(File classPath)
    {
        try {
            if (Util.isPackageDir(classPath)) {
                addClassPath(classPath.toURI().toURL());
                File[] fileList = classPath.listFiles();
                if (fileList == null) {
                    throw new RuntimeException("Invalid path: " + classPath);
                }
                for (File target : fileList) {
                    addClassPath(target);
                }
            }
        } catch (Throwable e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * jar ファイルが置かれているディレクトリを再帰的に辿り,
     * システムクラスローダーに追加していきます.
     */
    public static void addLibrary(File lib)
    {
        try {
            if (lib.isDirectory()) {
                File[] fileList = lib.listFiles();
                if (fileList == null) {
                    throw new RuntimeException("Invalid path: " + lib);
                }
                for (File target : fileList) {
                    addLibrary(target);
                }
            } else {
                String suffix = Util.getSuffix(lib.getName());
                if (suffix.equalsIgnoreCase("zip") || suffix.equalsIgnoreCase("jar")) {
                    addClassPath(lib.toURI().toURL());
                }
            }
        } catch (Throwable e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * システムクラスローダーにクラスパスを強制的に追加します.
     */
    public static void addClassPath(URL url)
    {
        URLClassLoader loader = (URLClassLoader) ClassLoader.getSystemClassLoader();
        Class<?> clazz = URLClassLoader.class;
        try {
            Method method = clazz.getDeclaredMethod("addURL", PARAMETERS);
            method.setAccessible(true);
            method.invoke(loader, new Object[]{url});
        } catch (NoSuchMethodException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }
}
