package com.perforb.jpaz.common;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.TreeMap;

import org.apache.commons.lang.StringUtils;

/**
 * Utility Class.
 * 
 * @author perforb
 */
public class Util
{
    /**
     * extension of class file.
     */
    private static final String SUFFIX_CLASS = ".class";

    /**
     * Private Constructor.
     */
    private Util()
    {
    }

    /**
     * Checks suffix of file name.
     */
    public static boolean isClassFile(String fileName)
    {
        if (StringUtils.isEmpty(fileName)) return false;
        return fileName.endsWith(SUFFIX_CLASS);
    }

    /**
     * Returns suffix of file name.
     */
    public static String getSuffix(String fileName)
    {
        if (StringUtils.isEmpty(fileName)) {
            return fileName;
        }
        int index = fileName.lastIndexOf('.');
        if (index >= 0) {
            return fileName.substring(index + 1);
        }
        return fileName;
    }

    /**
     * Removes suffix of file name.
     */
    public static String removeSuffix(String fileName)
    {
        if (StringUtils.isEmpty(fileName)) {
            return fileName;
        }
        int index = fileName.lastIndexOf('.');
        if (index >= 0) {
            return fileName.substring(0, index);
        }
        return fileName;
    }

    /**
     * Creates the package directory from classes
     * directory and package name.
     * 
     * @param classesDir
     * @param packageName Ex)com.perforb.jpaz
     */
    public static File getPackageDir(File classesDir, String packageName)
    {
        if (classesDir == null || packageName == null) return null;
        File packageDir = null;
        try {
            packageDir = new File(classesDir.getCanonicalPath());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        if (StringUtils.isNotEmpty(packageName)) {
            String[] names = packageName.split("\\.");
            for (String name : names) {
                packageDir = new File(packageDir, name);
            }
        }
        return packageDir;
    }

    /**
     * Checks package directory.
     */
    public static boolean isPackageDir(File target)
    {
        return target.exists() && target.isDirectory() && !target.getName().matches("^[\\.]+.*");
    }

    /**
     * If there is a tilde at the beginning of the
     * path, it is replaced with the home directory
     * of the user.
     * 
     * @return Replaced path
     */
    public static String replaceToHome(String path)
    {
        if (StringUtils.isEmpty(path) || !path.startsWith("~")) return path;
        return path.replaceFirst("~", System.getProperty("user.home"));
    }

    /**
     * Outputs System Property List to STDOUT.
     * This method is for debug.
     */
    public static void outputSystemProperties()
    {
        Properties properties = System.getProperties();
        Map<Object, Object> map = new TreeMap<Object, Object>();
        map.putAll(properties);
        for (Iterator<Entry<Object, Object>> it = map.entrySet().iterator(); it.hasNext();) {
            Entry<Object, Object> entry = it.next();
            System.out.println(entry.getKey() + "=" + entry.getValue());
        }
    }

    public static Class<?> forName(String className)
    {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        try {
            return Class.forName(className, true, loader);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(className, e);
        }
    }
}
