package com.perforb.jpaz.extract;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.cli.CommandLine;

import com.perforb.jpaz.common.ClassPathModifier;
import com.perforb.jpaz.common.Util;
import com.perforb.jpaz.extract.component.Component;
import com.perforb.jpaz.extract.component.Package;
import com.perforb.jpaz.print.ConsolePrinter;
import com.perforb.jpaz.print.CsvPrinter;
import com.perforb.jpaz.print.Printer;

public class Extracter
{
    public void execute(CommandLine line)
    {
        File classesDir = new File(Util.replaceToHome(line.getOptionValue("classesdir")));
        String libDir = Util.replaceToHome(line.getOptionValue("libdir", null));
        String rootPackage = line.getOptionValue("rootpackage");
        String outlim = line.getOptionValue("outlim");

        ClassPathModifier.addClassPath(classesDir);
        if (libDir != null) {
            ClassPathModifier.addLibrary(new File(libDir));
        }

        String[] packages = rootPackage.split(",");
        List<Component> components = new ArrayList<Component>();
        List<CalculationResult> results = new ArrayList<CalculationResult>();

        for (String pkg : packages) {
            Component root = new Package(pkg, true);
            root.build(classesDir, pkg, outlim);
            components.add(root);
            results.add(ComponentCalculator.getInstance().getResult(root));
        }

        CalculationResult sum = ComponentCalculator.getInstance().sumCalculationResult(
                results.toArray(new CalculationResult[0]));

        if (line.hasOption("console")) {
            PrintWriter writer = null;
            try {
                writer = new PrintWriter(System.out);
                Printer printer = new ConsolePrinter(writer);
                printer.printHeader(sum);
                for (int i = 0; i < packages.length; i++) {
                    printer.print(components.get(i), sum);
                }
            } finally {
                if (writer != null) {
                    writer.flush();
                    writer.close();
                }
            }
        }

        if (line.hasOption("csv")) {
            PrintWriter writer = null;
            try {
                String csvpath = Util.replaceToHome(line.getOptionValue("csv"));
                writer = getPrintWriter(csvpath, line.getOptionValue("charset", "UTF-8"));
                Printer printer = new CsvPrinter(writer);
                printer.printHeader(sum);
                for (int i = 0; i < packages.length; i++) {
                    printer.print(components.get(i), sum);
                }
            } finally {
                if (writer != null) {
                    writer.flush();
                    writer.close();
                }
            }
        }
    }

    protected PrintWriter getPrintWriter(String path, String charset)
    {
        PrintWriter writer = null;
        try {
            writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(
                    new FileOutputStream(path), charset)));
        } catch (Throwable e) {
            throw new RuntimeException(e);
        }
        return writer;
    }
}
