package com.perforb.jpaz.extract.component;

import java.io.File;
import java.lang.reflect.Method;
import java.util.List;

/**
 * 木構造のデータを表す Interface です. この Interface は
 * Composite Pattern における Component 役を担います.
 * 
 * @author perforb
 * @see Package
 * @see ClassFile
 */
public interface Component
{
    /**
     * 木構造のデータを構築します.本メソッドはルートとなる Object でのみ有効です.
     * 
     * @param classesDir
     * @param rootPackage
     * @param outlim
     * @return ビルドが成功した場合に true を返します.
     */
    boolean build(File classesDir, String rootPackage, String outlim);

    /**
     * {@link Package} Object の場合はパッケージ名,
     * {@link ClassFile} Object の場合はクラス名を返します.
     * 
     * @return
     */
    String getName();

    /**
     * パッケージ名を返します.
     * 
     * @return
     */
    String getPackageName();

    /**
     * {@link Component} Object を追加します.
     * 
     * @param component
     * @return
     */
    boolean addComponent(Component component);

    /**
     * {@link Component} Object を削除します.
     * 
     * @param component
     * @return
     */
    boolean removeComponent(Component component);

    /**
     * 子要素を返します.
     * 
     * @return
     */
    List<Component> getChildren();

    /**
     * クラスで定義されているメソッドを返します.
     * 
     * @return
     */
    Method[] getMethods();

    /**
     * 木構造のルートであれば true を返します.
     * 
     * @return
     */
    boolean isRoot();

    /**
     * {@link Package} Object であれば true を返します.
     * 
     * @return
     */
    boolean isPackage();

    /**
     * {@link ClassFile} Object であれば true を返します.
     * 
     * @return
     */
    boolean isClassFile();
}
