package com.perforb.jpaz.extract.component;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

/**
 * パッケージを表すクラスです. このクラスは Composite Pattern における
 * Composite 役を担います.
 * 
 * @author perforb
 * @see Component
 * @see ClassFile
 */
public class Package extends AbstractComponent
{
    /**
     * Leaf を保持する {@link List}.
     */
    private List<Component> components = new ArrayList<Component>();

    /**
     * Constructor.
     */
    public Package(String packageName)
    {
        super(packageName);
    }

    /**
     * For ROOT {@link Component} Constructor.
     * 
     * @param packageName パッケージ名
     * @param isRoot ROOT Component の場合に true をセット
     */
    public Package(String packageName, boolean isRoot)
    {
        super(packageName, isRoot);
    }

    @Override
    public String getPackageName()
    {
        return getName();
    }

    @Override
    public boolean addComponent(Component component)
    {
        return components.add(component);
    }

    @Override
    public boolean removeComponent(Component component)
    {
        return components.remove(component);
    }

    @Override
    public List<Component> getChildren()
    {
        return components;
    }

    @Override
    public Method[] getMethods()
    {
        return null;
    }

    @Override
    public boolean isPackage()
    {
        return true;
    }

    @Override
    public boolean isClassFile()
    {
        return false;
    }
}
