package com.perforb.jpaz.print;

import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import com.perforb.jpaz.extract.CalculationResult;
import com.perforb.jpaz.extract.component.Component;

public class ConsolePrinter extends AbstractPrinter
{
    public ConsolePrinter(PrintWriter writer) {
        super(writer);
    }

    @Override
    public void printHeader(final CalculationResult result)
    {
        int pkgLength = result.getMaxLengthOfPackageName();
        int classLength = result.getMaxLengthOfClassName();
        int packagePadding = (pkgLength < 9) ? 9 : pkgLength;
        int classPadding = (classLength < 7) ? 7 : classLength;
        writer.printf("Package: %d total, Class: %d total, Method: %d total\n",
                result.getTotalPackage(), result.getTotalClass(), result.getTotalMethod());
        writer.printf("%-" + packagePadding + "s ", "<Package>");
        writer.printf("%-" + classPadding + "s ", "<Class>");
        writer.printf("%s\n", "<Method>");
    }

    protected void printMethodList(final Component component, final CalculationResult result)
    {
        int pkgLength = result.getMaxLengthOfPackageName();
        int classLength = result.getMaxLengthOfClassName();
        int packagePadding = (pkgLength < 9) ? 9 : pkgLength;
        int classPadding = (classLength < 7) ? 7 : classLength;
        if (component.getMethods().length <= 0) {
            writer.printf("%-" + packagePadding + "s ", component.getPackageName());
            writer.printf("%-" + classPadding + "s ", component.getName());
            writer.printf("%s\n", "-");
            return;
        }
        for (Method m : component.getMethods()) {
            String mod = Modifier.toString(m.getModifiers());
            mod = (mod.length() > 0) ? mod + " " : mod;
            String returnType = m.getReturnType().getName();
            String name = m.getName();
            writer.printf("%-" + packagePadding + "s ", component.getPackageName());
            writer.printf("%-" + classPadding + "s ", component.getName());
            writer.printf("%s%s %s", mod, returnType, name);
            Class<?>[] paramList = m.getParameterTypes();
            writer.printf("%s", "(");
            for (int i = 0; i < paramList.length; i++) {
                writer.print(paramList[i].getName());
                if (!(i == paramList.length - 1)) writer.print(", ");
            }
            writer.printf("%s\n", ")");
        }
    }
}
