package com.perforb.jpaz.print;

import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import org.apache.commons.lang.StringUtils;

import com.perforb.jpaz.extract.CalculationResult;
import com.perforb.jpaz.extract.component.Component;

public class CsvPrinter extends AbstractPrinter
{
    private static final String LF        = System.getProperty("line.separator");
    private static final String ENCLOSURE = "\"";

    public CsvPrinter(PrintWriter writer) {
        super(writer);
    }

    @Override
    public void printHeader(final CalculationResult result)
    {
        StringBuffer buf = new StringBuffer();
        buf.append(String.format("Package: %d total, ", result.getTotalPackage()));
        buf.append(String.format("Class: %d total, ", result.getTotalClass()));
        buf.append(String.format("Method: %d total", result.getTotalMethod()));
        write(writer, buf.toString(), ENCLOSURE);
        write(writer, LF);
        write(writer, "Package", ENCLOSURE);
        write(writer, ",");
        write(writer, "Class", ENCLOSURE);
        write(writer, ",");
        write(writer, "Method", ENCLOSURE);
        write(writer, LF);
    }

    protected void printMethodList(final Component component, final CalculationResult result)
    {
        if (component.getMethods().length <= 0) {
            write(writer, component.getPackageName(), ENCLOSURE);
            write(writer, ",");
            write(writer, component.getName(), ENCLOSURE);
            write(writer, ",");
            write(writer, "-", ENCLOSURE);
            write(writer, LF);
            return;
        }
        StringBuffer buf = new StringBuffer();
        for (Method m : component.getMethods()) {
            if (buf.length() > 0) buf.setLength(0);
            String mod = Modifier.toString(m.getModifiers());
            mod = (mod.length() > 0) ? mod + " " : mod;
            String returnType = m.getReturnType().getName();
            String name = m.getName();
            write(writer, component.getPackageName(), ENCLOSURE);
            write(writer, ",");
            write(writer, component.getName(), ENCLOSURE);
            write(writer, ",");
            buf.append(String.format("%s%s %s", mod, returnType, name));
            Class<?>[] paramList = m.getParameterTypes();
            buf.append("(");
            for (int i = 0; i < paramList.length; i++) {
                buf.append(paramList[i].getName());
                if (!(i == paramList.length - 1)) buf.append(", ");
            }
            buf.append(")");
            write(writer, buf.toString(), ENCLOSURE);
            write(writer, LF);
        }
    }

    protected void write(PrintWriter writer, String value)
    {
        write(writer, value, null);
    }

    protected void write(PrintWriter writer, String value, String enclosure)
    {
        value = (value == null) ? "" : value;
        if (StringUtils.isEmpty(enclosure)) {
            writer.print(value);
        } else {
            writer.print(enclosure);
            writer.print(value);
            writer.print(enclosure);
        }
    }
}
